@@ -1,8 +1,8 @@
-import json
 import ast
 
 import click
-import os, shutil
+import os
+import shutil
 import re
 import sys
 import platform
@@ -10,6 +10,8 @@ import subprocess
 import site
 import webbrowser
 from datetime import datetime
+from json import load as json_load
+from json import dumps as json_dumps
 from dsframework.documentation.scripts import doxygen_handler
 
 isWindows = platform.system() == 'Windows'
@@ -35,6 +37,7 @@ directories = {
     'trainer': '',
     'server': '',
     'dsp': '',
+    'documentation': ''
 }
 
 ##
@@ -43,6 +46,7 @@ directories = {
 #        Create: Project/Forcer/Predictable/Tester-files/Deploy-files/Cloud-Eval-Files  \n
 #        Run   : Server / Evaluation (CSV,Cloud) \n
 
+
 class AliasedGroup(click.Group):
     """! AliasedGroup implements click.Group base class."""
 
@@ -55,63 +59,85 @@ class AliasedGroup(click.Group):
             pass
         return super().get_command(ctx, cmd_name)
 
+
 @click.group(cls=AliasedGroup)
 @click.version_option(package_name='dsFramework')
 def cli():
     """
     DS framework cli
 
-    ** how to use **
+    ** How to use **
 
     g = generate
 
-    create project
-
+    Create project
+    ==============
     dsf-cli g project my-new-project
 
-    dsf-cli generate project my-new-project
-
-    cd my-new-project
-
-    create forcer
-
+    Generate forcer
+    ===============
     dsf-cli g forcer my-new-forcer
 
-    create predictable
-
+    Generate predictable
+    ====================
     dsf-cli g predictable my-new-predictable
 
-    create tester files
-
-    dsf-cli create-tester-files
+    Create files
+    ===================
+    dsf-cli create-files [tester / deploy / cloud_eval / documentation / batch]
 
     run server
-
+    ==========
     dsf-cli run-server
 
-    create deploy files
-
-    dsf-cli create-deploy-files
-
-    create cloud eval files
-
-    dsf-cli create-cloud-eval-files
-
     run evaluation on a csv input
-
+    =============================
     dsf-cli evaluate input-csv-file batch-size
 
-    Generate project documentation:
+    ==============
+    Documentation
+    ==============
 
+    Generate documentation
+    ======================
     dsf-cli generate documentation full
 
     Generate documentation - without base classes:
-
+    ==============================================
     dsf-cli generate documentation simple
 
+    Show documentation
+    ==================
     dsf-cli show-documentation
 
+    Upload documentation
+    ====================
     dsf-cli upload-documentation
+
+    ========================
+    Batch project abilities
+    ========================
+
+    DataProc Cluster Manipulation
+    =============================
+    dsf-cli dataproc-cluster <create / stop / start / delete / connect>
+
+    Workflow Template Manipulation
+    ==============================
+    dsf-cli workflow-template <create / delete / instantiate>
+
+    DAG Manipulation
+    ================
+    dsf-cli dag <create / import / delete>
+
+    Upload files to bucket
+    ======================
+    dsf-cli upload-batch-files <jars, stages, whl , cluster_init, all>
+
+    Manage
+    Create sonar scanner properties file:
+
+    dsf-cli create-sonar-file current_project_repo_name
     """
 # @click.option("--type", prompt="type", help="type of component")
 # @click.option("--project_name", prompt="project_name", help="project_name")
@@ -122,7 +148,11 @@ def cli():
 @click.argument('gen_type')
 @click.argument('project_name')
 def generate(gen_type, project_name):
-    """List all cataloged APIs."""
+    """! Build a generation function from the incoming gen_type.
+        Args:
+            gen_type: A string containing one of the following: [project / forcer / predictable / documentation]
+            project_name: A string declaring our project name
+    """
     try:
         f = globals()["generate_%s" % gen_type]
     except Exception as e:
@@ -130,10 +160,24 @@ def generate(gen_type, project_name):
         return
     f(project_name)
 
+@cli.command()
+@click.argument('file_type')
+def create_files(file_type):
+    """! Build a file creation function from the incoming file type.
+        Args:
+            file_type: A string containing one of the following: [tester / deploy / cloud_eval / documentation / batch]
+    """
+    try:
+        f = globals()["create_%s_files" % file_type]
+    except Exception as e:
+        click.echo('type ' + file_type + ' not found')
+        return
+    f()
+
 
 @cli.command()
 def run_server():
-    """Using /server/main.py """
+    """! Start a uvicorn server by running /server/main.py """
     currentPipelineFolder = os.path.abspath(os.getcwd())
     currentParentFolder = os.path.join(currentPipelineFolder,"..")
     os.environ["PYTHONPATH"] = currentPipelineFolder
@@ -144,7 +188,7 @@ def run_server():
 @click.argument('tag')
 @click.argument('env', default="prd")
 def cloud_eval(tag, env):
-#    """Cloud evaluation running on AWS """
+    # """Cloud evaluation running on AWS """
     currentPipelineFolder = os.path.abspath(os.getcwd())
     currentParentFolder = os.path.join(currentPipelineFolder,"..")
     os.environ["PYTHONPATH"] = currentPipelineFolder
@@ -176,7 +220,7 @@ def cloud_eval(tag, env):
                 # print('selected_dataset', selected_dataset)
                 folder = currentPipelineFolder + '/tester/dsp/add_experiment.py'
                 subprocess.call(('dvc_push.sh'), shell=True)
-                output = subprocess.call(('python', folder, env, json.dumps(model), json.dumps(selected_dataset), tag), shell=True)
+                output = subprocess.call(('python', folder, env, json_dumps(model), json_dumps(selected_dataset), tag), shell=True)
                 print('you can see your experiment in que')
             else:
                 print('selection does not exist')
@@ -186,26 +230,151 @@ def cloud_eval(tag, env):
         print('could not get models')
 
 
-@cli.command()
 def create_deploy_files():
-    """Create deploy files (created through 'Create project', but we can use it as standalone function """
+    """! Create deploy files (created through 'Create project', but can be used as a standalone function) """
     copy_deploy_files('')
 
 
-@cli.command()
 def create_cloud_eval_files():
-    """Create cloud evaluation files (created through 'Create project', but we can use it as standalone function """
+    """! Create cloud evaluation files (created through 'Create project', but can be used as a standalone function) """
     currentPipelineFolder = os.path.abspath(os.getcwd())
     copy_cloud_eval_files(currentPipelineFolder)
 
 
-@cli.command()
 def create_tester_files():
-    """!Create tester files (created through 'Create project', but there is an option
-       We can use it as standalone function """
+    """! Create tester files (created through 'Create project', but can be used as a standalone function) """
     create_tester_env()
 
 
+def create_documentation_files():
+    """! Create documentation files (created through 'Create project', but can be used as a standalone function) """
+    project_name = os.path.basename(os.getcwd())
+    copy_documentation_files(os.path.join(directories['main'], 'documentation'))
+    update_doc_configuration(project_name, os.path.join(directories['main'], 'documentation'))
+
+
+def create_batch_files():
+    """! Create batch files (created through 'Create project', but can be used as a standalone function) """
+    copy_batch_files(directories['main'])
+
+
+@cli.command()
+@click.argument('action')
+def dataproc_cluster(action: str):
+    """! Handle cluster action, either create,  start, stop, delete, or connect to the Dataproc cluster
+            Args:
+                 action: an action to perform, chosen from: {create, start, stop, delete, connect}
+    """
+    valid_actions = {'create', 'start', 'connect', 'stop', 'delete'}
+    if action not in valid_actions:
+        print(f"Can't determine file type {action}. Please use one of: {valid_actions}")
+        return
+
+    batch_helper = config_batch_helper()
+
+    if action == 'create':
+        # Create DataProc cluster according to project specific configuration
+        batch_helper.create_dataproc_cluster()
+    elif action == 'start':
+        # Start an existing DataProc cluster according to project specific configuration
+        batch_helper.start_dataproc_cluster()
+    elif action == 'connect':
+        # Connect via ssh to an existing DataProc cluster according to project specific configuration.
+        # Connection will be made to the master node
+        batch_helper.connect_dataproc_cluster()
+    elif action == 'stop':
+        # Stop an existing DataProc cluster according to project specific configuration
+        batch_helper.stop_dataproc_cluster()
+    elif action == 'delete':
+        # Delete an existing DataProc cluster according to project specific configuration.
+        # Since we can't update certain parameters on an existing cluster, we need the option
+        # to delete it
+        batch_helper.delete_dataproc_cluster()
+
+
+@cli.command()
+@click.argument('action')
+def workflow_template(action: str):
+    """! Handle workflow action, either create, instantiate or delete the template
+        Args:
+             action: an action to perform, chosen from: {create, instantiate, delete}
+    """
+    valid_actions = {'create', 'delete', 'instantiate'}
+    if action not in valid_actions:
+        print(f"Can't determine action type: {action}. Please use one of: {valid_actions}")
+        return
+
+    current_folder = os.path.abspath(os.getcwd())
+
+    os.environ["PYTHONPATH"] = current_folder
+    workflow_folder = current_folder + '/batch_files/workflow/workflow.py'
+    basic_command = 'python ' + workflow_folder
+
+    print(f"Attempting to {action} a workflow template")
+    subprocess.call(basic_command + ' ' + action, shell=True)
+
+
+@cli.command()
+@click.argument('action')
+def dag(action: str):
+    """! Handle DAG actions, either create, delete, or import a DAG
+    Once imported, it will start as paused and will be set up for a daily activation
+        Args:
+             action: an action to perform, chosen from: {create, import, delete}
+    """
+    valid_actions = {'create', 'import', 'delete'}
+    if action not in valid_actions:
+        print(f"Can't determine action type: {action}. Please use one of: {valid_actions}")
+        return
+
+    current_folder = os.path.abspath(os.getcwd())
+
+    os.environ["PYTHONPATH"] = current_folder
+    dag_folder = current_folder + '/batch_files/workflow_dag/dag.py'
+    basic_command = 'python ' + dag_folder
+
+    print(f"Attempting to {action} a DAG")
+    subprocess.call(basic_command + ' ' + action, shell=True)
+
+
+@cli.command()
+@click.argument('file_type')
+def upload_batch_files(file_type: str):
+    """! Upload all files needed for a batch project to the predefined bucket
+        Args:
+            file_type: String containing the types of files to upload (jars, stages, whl , cluster_init, or "all")
+    """
+    batch_helper = config_batch_helper()
+    batch_helper.upload_files_to_bucket(file_type)
+
+
+def config_batch_helper():
+    """! Configure the batch file helper for a variety of cli actions
+        Returns:
+            ZIDSBatchHelper: Instantiation of the batch helper class
+    """
+    from dsframework.cli.batch.batch_helper_functions import ZIDSBatchHelper
+    current_folder = os.path.abspath(os.getcwd())
+    batch_helper_cls = None
+    with open(current_folder + '/batch_files/batch_config.json') as config_file:
+        config_dict = json_load(config_file)
+        batch_helper_cls = ZIDSBatchHelper(config_dict)
+    return batch_helper_cls
+
+
+@cli.command()
+@click.argument('repo_name')
+def create_sonar_file(repo_name: str):
+    """! Create a sonar scanner file.
+
+    Args:
+        repo_name: name of your repository.
+    """
+    current_folder = os.path.abspath(os.getcwd())
+    data = "sonar.projectKey=dozi_" + repo_name
+    create_file(os.path.join(current_folder, "sonar-scanner.properties"), data)
+
+
 @cli.command()
 @click.argument('csv_file_path')
 @click.argument('batch_size')
@@ -521,6 +690,7 @@ def create_project(project_name):
     create_server_config_json()
     copy_deploy_files(directories['main'])
     copy_cloud_eval_files(directories['main'])
+    copy_batch_files(directories['main'])
     copy_documentation_files(os.path.join(directories['main'], 'documentation'))
     update_doc_configuration(project_name, os.path.join(directories['main'], 'documentation'))
 
@@ -648,27 +818,34 @@ def create_exist_pipeline_file(type, fileName):
 
 
 def create_tester_env():
-    """
-    This function will create a testing environment, if one does not already exist in the project.
+    """! This function will create a testing environment, if one does not already exist in the project.
     It will create the testing, evaluation, and relevant schema files.
     """
 
     project_name = os.getcwd().split('/')[-1]
     dst_folder = 'tester/'
     schema_dst_folder = 'tester/test_schema/'
+    dsp_dst_folder = 'tester/dsp/'
     tester_file_list = ["tester/general_tester",
                         "tester/evaluator",
                         "schema/test_input",
-                        "schema/test_output"]
+                        "schema/test_output",
+                        "tester/dsp/add_experiment",
+                        "tester/dsp/get_dataset_options",
+                        "tester/dsp/get_model_options"]
 
     if not os.path.exists(dst_folder):
         os.mkdir(dst_folder)
         os.mkdir(schema_dst_folder)
+        os.mkdir(dsp_dst_folder)
 
         for filename_with_path in tester_file_list:
             file_only = filename_with_path.split('/')[-1]
-            if file_only.startswith('test'):
+            parent_folder = filename_with_path.split('/')[-2]
+            if parent_folder == 'schema':
                 dst_folder = schema_dst_folder
+            elif parent_folder == 'dsp':
+                dst_folder = dsp_dst_folder
 
             data = read_template_file(filename_with_path)
 
@@ -1075,6 +1252,59 @@ def copy_cloud_eval_files(main_dir):
             print(f"Existing: [{conflicted_file[0]}] \t\t Framework source file [{conflicted_file[1]}]")
 
 
+def copy_batch_files(main_dir):
+    """! Generate batch files needed for batch projects. Creates 3 stages:
+    1. Get Data
+    2. Run Pipeline
+    3. Set Data
+
+    Along with a matching workflow file, and a configuration file
+
+        Args:
+            main_dir: A string describing the main working directory, on existing projects, it's empty.
+    """
+
+    current_folder = os.path.basename(os.getcwd())
+    if main_dir != '':
+        current_folder = main_dir
+    service_name = current_folder.split('/')[-1]
+    project_name_no_underscore = clean_existing_project_name(service_name)
+    source_dir = os.path.join(__location__, 'dsframework/cli/tester/batch_files/')
+
+    for (dir_path, dir_name, filenames) in os.walk(source_dir):
+        dir_path = dir_path.replace(source_dir, '')
+        file_list = [os.path.join(dir_path, file) for file in filenames]
+        for file_path in file_list:
+            with open(os.path.join(source_dir, file_path), 'r', encoding="utf8") as file:
+                try:
+                    data = file.read()
+                    data = data.replace('{name-your-service}', service_name)
+                    data = data.replace('generatedProjectName', project_name_no_underscore)
+
+                    if main_dir == '':
+                        target_dir = 'batch_files/'
+                        dir_to_create = target_dir
+                    else:
+                        target_dir = current_folder + '/batch_files/'
+                        dir_to_create = target_dir
+
+                    if dir_path != "":
+                        dir_to_create = target_dir + dir_path + '/'
+
+                    if not os.path.exists(dir_to_create):
+                        os.makedirs(dir_to_create)
+                    if not os.path.exists(target_dir + file_path):
+                        f = open(target_dir + file_path, "w")
+                        f.write(data)
+                        f.close()
+                except Exception as ex:
+                    if file_path.rsplit('.', 1)[-1] == 'jar':
+                        shutil.copy(os.path.join(source_dir, file_path), f'{dir_to_create}')
+                    else:
+                        print(f'Error when trying to open: {file_path}: {ex}')
+                    pass
+
+
 def copy_documentation_files(target_dir):
     """! Copy documentation folder to newly created project.
 
