@@ -95,13 +95,11 @@ class ZIDS_SharedArtifacts():
                 type: file format, format supported: json, csv, pickle, tensorflow, key_value and key_value_int
                     additional formats can be added by overriding extend_load_file_type method.
         """
-        file_path = sys.modules[self.__class__.__module__].__file__
-        curr_path = file_path[:file_path.index("pipeline")]
         file_type = item['type']
         path = item['path']
         name = item['name']
         if path:
-            absolute_path = curr_path + '/' + path
+            absolute_path = self.base_dir + '/' + path
             if file_type == 'json':
                 self.load_json(absolute_path, name)
             elif file_type == 'csv':
@@ -219,4 +217,4 @@ class ZIDS_SharedArtifacts():
             return self.__dict__.update({key: val})
         return default
     ##
-    # @endcond
\ No newline at end of file
+    # @endcond
