@@ -18,15 +18,10 @@ fi
 if git rev-parse $VERSION >/dev/null 2>&1
 then
     echo "Found tag"
-    echo pushing to DVC
-    dvc push --remote ds-ml-artifacts-s3
 else
     echo "Tag not found, creating and pushing git tag "${VERSION}"..."
     git tag -a "${VERSION}" -m "${DESCRIPTION}"
     git config --global push.default current
     git push --follow-tags origin
-    # push to s3 dvc
-    echo pushing to DVC
-    dvc push --remote ds-ml-artifacts-s3
 fi
 
