@@ -4,11 +4,13 @@ import logging
 import sys
 from pythonjsonlogger import jsonlogger
 
+##  Logger setup
 root = logging.getLogger()
 root.setLevel(logging.INFO)
-
+## Redirect logging to sys.stdout
 handler = logging.StreamHandler(sys.stdout)
 handler.setLevel(logging.INFO)
+## Assign logger a json formatter to output log data as json.
 formatter = jsonlogger.JsonFormatter()
 handler.setFormatter(formatter)
 root.addHandler(handler)
@@ -50,14 +52,25 @@ def load_allowed_cors_origins():
 
     return allowed_origins
 
-
+## Instantiate FastAPI()
 app = FastAPI()
-
+## Allowed cors origins
 origins = load_allowed_cors_origins()
+## Methods allowed, for FastAPI()
 methods = ["*"]
+## Headers allowed, for FastAPI()
 headers = ["*"]
+## Credentials required, bool.
 credentials = True
-
+##
+# @var allow_origins
+# Cors allowed origins.
+# @var allow_credentials
+# Allowed credentials by FastAPI()
+# @var allow_methods
+# Allowed methods by FastAPI()
+# @var allow_headers
+# Allowed headers by FastAPI()
 app.add_middleware(
     CORSMiddleware,
     allow_origins=origins,
@@ -65,7 +78,7 @@ app.add_middleware(
     allow_methods=methods,
     allow_headers=headers,
 )
-
+## Initialized a pipeline for /predict /parse endpoints.
 pipeline = generatedProjectNamePipeline()
 # from server.pool import InstancePool
 # pipelines = [generatedProjectNamePipeline() for i in range(3)]
@@ -74,9 +87,17 @@ pipeline = generatedProjectNamePipeline()
 
 @app.post('/predict', dependencies=[Depends(get_token_header)])
 def predict(body: generatedProjectNameInputs, request: Request) -> List[generatedProjectNameOutputs]:
-    '''
-    predict api
-    '''
+    """! Predict api endpoint, is designed to support predict projects such as
+    scoops classification, valid user email etc.
+
+    Use by post request, for example:
+
+        headers = {"Content-Type": "application/json; charset=UTF-8", **global_headers}
+        url = http://localhost:8080/predict
+        data = {}  # The data to run by model.
+        response = requests.post(url, json=data, headers=headers)
+
+    """
     data = body.dict()
     # logger.info({"message": "predict invoked", "data": json.dumps(data)})
     # print('data', data)
@@ -95,9 +116,16 @@ def predict(body: generatedProjectNameInputs, request: Request) -> List[generate
 
 @app.post('/parse', dependencies=[Depends(get_token_header)])
 def parse(body: generatedProjectNameInputs, request: Request) -> List[generatedProjectNameOutputs]:
-    '''
-    parse api
-    '''
+    """! Parse api endpoint, is designed to support parse projects such as trex, bio parser, sig parser etc.
+
+    Use by post request, for example:
+
+        headers = {"Content-Type": "application/json; charset=UTF-8", **global_headers}
+        url = http://localhost:8080/parse
+        data = {}  # The data to run by model.
+        response = requests.post(url, json=data, headers=headers)
+
+    """
     data = body.dict()
     # logger.info({"message": "parse invoked", "data": json.dumps(data)})
     # print('data', data)
@@ -114,21 +142,35 @@ def parse(body: generatedProjectNameInputs, request: Request) -> List[generatedP
     return output
 
 
-# Fetch the proper tester for the project
+## Fetch the proper tester for the project
 model_tester = generatedProjectNameGeneralTester()
 
 
 @app.post('/test', dependencies=[Depends(get_token_header), Depends(verify_env)], include_in_schema=False)
 def test(input_request: TestInputRequest) -> List[TestOutputResponse]:
-    """
-    This is the main endpoint for testing. Every project starts with a mock response service, in order to enable
-    DSP integration, but as soon as a working testing functions are ready, please replace the mock service with the
-    real one.
+    """! This is the main endpoint for testing.
+
+        Args:
+            input_request: Request as received from the DSP
+        Returns:
+            A list of responses, to be sent back to the DSP
+
+    Every project starts with a default mock response service, in order to enable DSP integration (when the working
+    testing functions are ready and implemented in the tester/general_tester.py),
+    replace the mock service with the real one, in practice just remark the following line:
+
+        response = model_tester.create_mock_response(input_request)
+
+    and remove remark quotes from the following block of code:
+
+        model_tester.save_meta_data(input_request)
+        response = []
+        # Until this model will support batches, work sequentially. Once it does, pass the full list of rows.
+        for request_row in input_request.rows:
+            input_list = [request_row]
+            tester_output = model_tester.test_batch(input_list)
+            response.append(tester_output[0])
 
-    Parameters:
-        input_request: Request as received from the DSP
-    Returns:
-        A list of responses, to be sent back to the DSP
     """
     response = model_tester.create_mock_response(input_request)
 
@@ -146,14 +188,38 @@ def test(input_request: TestInputRequest) -> List[TestOutputResponse]:
 
 @app.get("/livenessprobe")
 def liveness_probe():
+    """! Endpoint used to test server liveness
+
+        Usage:
+            http://localhost:8080/livenessprobe
+
+        Returns:
+            server response: {"alive": True} - On success
+
+    """
     return {"alive": True}
 
 
 @app.get("/readinessprobe")
 def readiness_probe():
+    """! Endpoint used to test server readiness
+
+        Usage:
+            http://localhost:8080/readinessprobe
+
+        Returns:
+            server response: {"ready": True} - On success
+
+        """
     return {"ready": True}
 
 
 if __name__ == '__main__':
+    # Runs the server
+    ##
+    # @var host
+    # IP Address
+    # @var port
+    # Port number
     uvicorn.run(app, host='0.0.0.0', port=8080)
 
